/*
 * UNIVERSITE PARIS 1 (PANTHEON SORBONNE)
 * MIAGE - UFR 27
 */
package review;

import java.util.Arrays;

/**
 * Cette classe est un exemple de violation du principe de l'encapsulation. 
 * Elle attribue un accès direct à ses structures internes à travers la
 * méthode getValues et par une visibilité package. 
 * 
 * @author Manuele Kirsch Pinheiro <kirschpm@gmail.com>
 */
public class Etudiant {

    public static final int DEFAULT_SIZE = 4;
    
    String nom;
    private int values[]; //notes de l'étudiant
    int dernier; //dernière note ajoutée
    int max; //nb de notes possibles

    /**
     * 
     * @param name : nom de l'étudiant
     * @param size : combien de notes on peut attribuer à cet étudiant
     */
    public Etudiant(String name, int size) {
        this.nom = name;
        this.dernier = 0;
        if (size > 0) {
            this.max = size;
        } else {
            this.max = Etudiant.DEFAULT_SIZE;
        }

        this.values = new int[max];
    }

    public String getNom() {
        return nom;
    }

    public void setNom(String nom) {
        this.nom = nom;
    }
    
    /**
     * ajoute une nouvelle note à l'étudiant
     * @param v valeur de la nouvelle note
     * @return true si on a pu ajouter la nouvelle note
     *         false si toutes les notes prévues avaient déjà été renseignées 
     */
    public boolean addValue (int v) {
        boolean ok = false;
        
        if(dernier<max) {
            this.values[dernier++] = v;
            ok = true;
        }
        
        return ok;
    }
    
    /**
     * methode qui viole l'encapsulation de la classe.
     * Il retourne une référence directe à la structure interne. 
     * N'importe qui peut alors la modifier. 
     */
    public int[] getValues() {
        return this.values;
    }
    
    /**
     * calcule la moyenne des notes de l'étudiant
     * @return 
     */
    public int average() {
        int somme=0;
        
        if (dernier>0) {
            for (int i: this.values) {
                somme+= i;
            }
        }
        
        return (dernier>0)?somme/dernier:0;
    }
    
}
