/*
 * UNIVERSITE PARIS 1 (PANTHEON SORBONNE)
 * MIAGE - UFR 27
 */
package review.exercices;

/**
 * Classe représentant un tableau d'objets, dans lequel l'accès aux cases
 * est directe et limité à une seule classe d'objets, quelque soit cette classe.  
 * Il s'agit d'un exemple d'usage des génériques en Java. 
 * @author Manuele Kirsch Pinheiro <kirschpm@gmail.com>
 */
public class TableauObjetGenerique<T> {
    /** taille de tableau par défaut */
    public static int DEFAULT_SIZE=10; 
    private T[] tableau;
    private int taille;

    /**
     * construction d'un objet TableauObjetParametre d'une taille donnée. 
     * Si celle-ci ne correspond pas à une taille valable (size<0 par exemple), 
     * le tableau aura une taille par défaut. 
     * @param size taille du nouveau tableau d'objets
     * @see #DEFAULT_SIZE
     */
    public TableauObjetGenerique(int size) {
        if (size>0)
            this.taille=size;
        else
            this.taille = DEFAULT_SIZE;
        
        //impossible ! limitation java
        //this.tableau = new T[this.taille];
        this.tableau = (T[])new Object[this.taille];
    }

    /**
     * ajoute une nouvelle valeur à une case du tableau. Si la case est occupée,
     * l'ancienne valeur de la case est remplacée. 
     * @param pos position où la valeur sera ajoutée
     * @param value valeur à ajouter
     * @throws IndexOutOfBoundsException si position st invalide (négative ou 
     * dépassant la tailleu du tableau)
     */
    public void fillCase(int pos, T value) throws IndexOutOfBoundsException{
        if (pos>=0 && pos<this.taille) { //on verifie la taille
            this.tableau[pos]=value;
        }
        else {
            //la position est invalide, on lance alors une exception
            throw new IndexOutOfBoundsException("Position " + pos
                    + " invalide ou inoccupée");
        }
    }

    /**
     * récupère la valeur stockée dans l'indice (position) indiqué en paramètre.
     * @param pos position de la valeur à récupérer
     * @return valeur dans la position pos ou null, si la case est vide
     */
    public T getCase(int pos) {
        if (pos>=0 && pos<this.taille) 
            return this.tableau[pos];
        else
            return null;
    }

    /**
     * nettoie la valeur de la case.
     * @param pos position de la case à nettoyer
     */
    public void clearCase(int pos) {
        if (pos>=0 && pos<this.taille) 
            this.tableau[pos]=null;
    }

    /**
     * recupère la taille du tableau
     * @return 
     */
    public int size() {
        return this.taille;
    }
}