/*
 * UNIVERSITE PARIS 1 (PANTHEON SORBONNE)
 * MIAGE - UFR 27
 */
package review.exercices.business;

import java.sql.Statement;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.text.MessageFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.Iterator;
import review.business.Employee;

/**
 * Exemple de modèle lisant les informations à partir d'une base de données
 * @author kirsch
 */
public class DataBaseDepartmentModel extends BasicDepartmentModel {
    /* 
     * CES VARIABLES STATIQUES DEVAIENT ÊTRE LUES A PARTIR D'UN FICHIER
     * PROPERTIES STANDARD
     */
    //donnes de connexion par default

    private static final String DEFAULT_DRIVER = "org.apache.derby.jdbc.ClientDriver";
    private static final String DEFAULT_BDD = "jdbc:derby://localhost:1527/DepartmentBase";
    private static final String DEFAULT_LOGIN = "dept";
    private static final String DEFAULT_PWD = "dept";
    //attributs de tables dans la bd
    private final String SQLColEmployeeName = "name";
    private final String SQLColEmployeeSalary = "salary";
    private final String SQLColEmployeehire = "hire";
    private final String SQLColDeptName = "dname";
    //requetes SQL a executer
    private final String SQLEmployees = "select * from DEPT.EMPLOYEE";
    private final String SQLDepartment = "select dname from DEPT.DEPARTMENT";
    private String patternIns = "INSERT INTO DEPT.EMPLOYEE ( "
            + SQLColEmployeeName + ", " + SQLColEmployeeSalary + ", "
            + SQLColEmployeehire + ", " + SQLColDeptName
            + ") VALUES (''{0}'', {1}, ''{2}'', ''{3}'')";
    private String patternUp = "UPDATE DEPT.EMPLOYEE SET "
            + SQLColEmployeeSalary + " = {1}, "
            + SQLColEmployeehire + " = ''{2}'', "
            + SQLColDeptName + " = ''{3}'' WHERE "
            + SQLColEmployeeName + " = ''{0}''";
    private String patternSel = "SELECT * FROM DEPT.EMPLOYEE WHERE NAME = ''{0}''";
    private String patternInsDept = "INSERT INTO DEPT.DEPARTMENT ( "
            + SQLColDeptName + ") VALUES (''{0}'')";
    private String patternUpDept = "UPDATE DEPT.DEPARTMENT SET  "
            + SQLColDeptName + " = ''{0}''";
    private String patternSelDept = "SELECT * FROM DEPT.DEPARTMENT WHERE DNAME = ''{0}''";
    
    
    //BUGFIX
    //on avait oublié de supprimer un employé... oups... :p
    //DELETE FROM DEPT.EMPLOYEE WHERE name = '...';
    private String patternDel = "DELETE FROM DEPT.EMPLOYEE WHERE "
            + SQLColEmployeeName + " = ''{0}''"; 
    
    
    
    //variables permettant la connexion au BD
    private String driver;
    private String bdd;
    private String login;
    private String pass;
    private Connection conn;

    public DataBaseDepartmentModel() {
        super();
        this.driver = DEFAULT_DRIVER;
        this.bdd = DEFAULT_BDD;
        this.login = DEFAULT_LOGIN;
        this.pass = DEFAULT_PWD;
        this.conn = null;
    }

    public DataBaseDepartmentModel(String driver, String bdd, String login, String pass) {
        super();
        this.driver = driver;
        this.bdd = bdd;
        this.login = login;
        this.pass = pass;
        this.conn = null;
    }

    //BUGFIX 
    @Override
    public void removeEmployee(Employee e) {
        super.removeEmployee(e);
        
        String sql = MessageFormat.format(patternDel, e.getName()); 
        System.out.println(sql);
        
        if (this.conn != null) {            
            try {
                    Statement stat = conn.createStatement();
                    int nbReg = stat.executeUpdate(sql);
                    System.out.println("Employee deleted: " + nbReg);
                } catch (SQLException ex) {
                    //ICI ON DEVRAIT LANCER UNE EXCEPTION POUR AVERTIR LES AUTRES ELEMENTS,
                    //ET NOTAMMENT LES CONTROLEURS, QU'IL Y A EU UN SOUCI, AFIN QU'UN MESSAGE D'ERREUR
                    //SOIT PRESENTER A L'UTILSATEUR
                    ex.printStackTrace();
                }
        }
    }
    
    

    /**
     * enregistre les informations sur le département. 
     * Attention : rien ne sera enregistré si le département n'a pas été 
     * correctement lu (méthode load) auparavant. 
     * @see load
     */
    @Override
   public void save() { 
        //si on n'a fait un load avant, on n'a pas de connexion ouverte
        //donc on a rien à enregistrer
        if (this.conn != null) {
            this.saveDeptInfo();
            this.saveEmployees();
            this.closeConnection();
        }
    }

    /**
     * lit les informations concernant le département à partir de la base de données
     */
    @Override
    public void load() {
        this.openConnection();
        this.loadEmployees();
        this.loadDepartmentInfo();
    }
    
    protected void closeConnection() {
        if (conn != null) {
            try {
                conn.close();
                System.out.println("Cloture de la connexion à la BDD");
            } catch (SQLException ex) {
                //ICI ON DEVRAIT LANCER UNE EXCEPTION POUR AVERTIR LES AUTRES ELEMENTS,
                //ET NOTAMMENT LES CONTROLEURS, QU'IL Y A EU UN SOUCI, AFIN QU'UN MESSAGE D'ERREUR
                //SOIT PRESENTER A L'UTILSATEUR
                ex.printStackTrace();
            }
        }
    }

    protected void openConnection() {

        if (this.conn != null) {
            //d'abord on s'assure qu'il n'y a pas de connexion ouverte
            this.closeConnection();
        } else {
            try {
                //Charge le pilote
                Class.forName(driver);
                //Crée la connection
                conn = DriverManager.getConnection(bdd, login, pass);
                System.out.println("Connexion à la BDD");
            } catch (ClassNotFoundException cnfe) {
                this.conn = null;
                //ICI ON DEVRAIT LANCER UNE EXCEPTION POUR AVERTIR LES AUTRES ELEMENTS,
                //ET NOTAMMENT LES CONTROLEURS, QU'IL Y A EU UN SOUCI, AFIN QU'UN MESSAGE D'ERREUR
                //SOIT PRESENTER A L'UTILSATEUR
                cnfe.printStackTrace();
            } catch (SQLException sqle) {
                this.conn = null;
                //ICI ON DEVRAIT LANCER UNE EXCEPTION POUR AVERTIR LES AUTRES ELEMENTS,
                //ET NOTAMMENT LES CONTROLEURS, QU'IL Y A EU UN SOUCI, AFIN QU'UN MESSAGE D'ERREUR
                //SOIT PRESENTER A L'UTILSATEUR
                sqle.printStackTrace();
            }
        }
    }

    protected void loadEmployees() {
        if (this.conn != null) {
            try {
                System.out.println(this.SQLEmployees);

                Statement stat = conn.createStatement();
                ResultSet rs = stat.executeQuery(this.SQLEmployees);

                while (rs.next()) {
                    String name = rs.getString(this.SQLColEmployeeName);
                    double sal = rs.getDouble(this.SQLColEmployeeSalary);
                    Date hire = rs.getDate(this.SQLColEmployeehire);

                    Employee e = new Employee(name, (float) sal, hire);
                    this.addEmployee(e);
                }
            } catch (SQLException ex) {
                //ICI ON DEVRAIT LANCER UNE EXCEPTION POUR AVERTIR LES AUTRES ELEMENTS,
                //ET NOTAMMENT LES CONTROLEURS, QU'IL Y A EU UN SOUCI, AFIN QU'UN MESSAGE D'ERREUR
                //SOIT PRESENTER A L'UTILSATEUR
                ex.printStackTrace();
            }
        }
    }

    protected void loadDepartmentInfo() {
        if (this.conn != null) {
            try {
                Statement stat = conn.createStatement();
                ResultSet rs = stat.executeQuery(this.SQLDepartment);

                while (rs.next()) {
                    String name = rs.getString(this.SQLColDeptName);
                    this.dept.setName(name);
                }
            } catch (SQLException ex) {
                //ICI ON DEVRAIT LANCER UNE EXCEPTION POUR AVERTIR LES AUTRES ELEMENTS,
                //ET NOTAMMENT LES CONTROLEURS, QU'IL Y A EU UN SOUCI, AFIN QU'UN MESSAGE D'ERREUR
                //SOIT PRESENTER A L'UTILSATEUR
                ex.printStackTrace();
            }
        }
    }

    protected void saveEmployees() {

        if (this.conn != null) {
            Iterator<Employee> it = this.employees();

            while (it.hasNext()) {
                Employee e = it.next();

                //insert ou update?
                String sql = MessageFormat.format(patternSel, e.getName()); 
                String pattern = (this.exists(sql)) ? patternUp : patternIns;

                sql = MessageFormat.format(pattern, e.getName(),
                        Float.toString(e.getSalary()), this.buildDate(e), this.dept.getName());
                System.out.println(sql);
                try {
                    Statement stat = conn.createStatement();
                    stat.executeUpdate(sql);
                } catch (SQLException ex) {
                    //ICI ON DEVRAIT LANCER UNE EXCEPTION POUR AVERTIR LES AUTRES ELEMENTS,
                    //ET NOTAMMENT LES CONTROLEURS, QU'IL Y A EU UN SOUCI, AFIN QU'UN MESSAGE D'ERREUR
                    //SOIT PRESENTER A L'UTILSATEUR
                    ex.printStackTrace();
                }
            }
        }
    }

    protected void saveDeptInfo() {

        String sql = MessageFormat.format(patternSelDept, this.dept.getName());

        boolean exists = this.exists(sql);

        if (this.conn != null) {
            sql = MessageFormat.format((exists) ? patternUpDept : patternInsDept, this.dept.getName());
            System.out.println(sql);

            try {
                Statement stat = conn.createStatement();
                stat.executeUpdate(sql);
            } catch (SQLException ex) {
                //ICI ON DEVRAIT LANCER UNE EXCEPTION POUR AVERTIR LES AUTRES ELEMENTS,
                //ET NOTAMMENT LES CONTROLEURS, QU'IL Y A EU UN SOUCI, AFIN QU'UN MESSAGE D'ERREUR
                //SOIT PRESENTER A L'UTILSATEUR
                ex.printStackTrace();
            }
        }
    }

    protected boolean exists(String sql) {
        int n = 0;
        if (this.conn != null) {
            try {
                Statement stat = conn.createStatement();
                ResultSet rs = stat.executeQuery(sql);

                while (rs.next()) {
                    n++;
                }
            } catch (SQLException ex) {
                //ICI ON DEVRAIT LANCER UNE EXCEPTION POUR AVERTIR LES AUTRES ELEMENTS,
                //ET NOTAMMENT LES CONTROLEURS, QU'IL Y A EU UN SOUCI, AFIN QU'UN MESSAGE D'ERREUR
                //SOIT PRESENTER A L'UTILSATEUR
                ex.printStackTrace();
            }
        }
        return (n > 0) ? true : false;
    }

    private String buildDate(Employee e) {

        StringBuilder hiredate = new StringBuilder();

        Calendar cal = Calendar.getInstance();
        cal.setTime(e.getHireDay());

        int y = cal.get(Calendar.YEAR);
        int m = cal.get(Calendar.MONTH) + 1; //les mois sont compte a partir de 0
        int d = cal.get(Calendar.DAY_OF_MONTH);

        hiredate.append(y);
        hiredate.append("-");
        hiredate.append(m);
        hiredate.append("-");
        hiredate.append(d);

        return hiredate.toString();
    }
}
