/*
 * UNIVERSITE PARIS 1 (PANTHEON SORBONNE)
 * MIAGE - UFR 27
 */
package review.exercices.business;

import javax.swing.event.EventListenerList;
import review.business.Employee;

/**
 * Nouveau modèle représentant un département qui est enregistré sur une 
 * base de données. Seule modification par rapport au précédent modèle :
 * traitement d'un événement supplémentaire, indiquant les modifications qui
 * touchent un employé précis (ajout, modification, suppression). 
 * 
 * @author Manuele Kirsch Pinheiro <kirschpm@gmail.com>
 */
public class FullDataBaseDepartmentModel extends DataBaseDepartmentModel {
    private EventListenerList employee_listeners;
    
    public FullDataBaseDepartmentModel () {
        super();
        this.employee_listeners = new EventListenerList();
    }
    
    public FullDataBaseDepartmentModel (String driver, String bdd, String login, String pass) {
        super(driver, bdd, login, pass);
        this.employee_listeners = new EventListenerList();
    }
    
    public void addFullDepartmentListener(FullDepartmentListener l) {
        this.employee_listeners.add(FullDepartmentListener.class, l);
    }

    public void removeFullDepartmentListener(FullDepartmentListener l) {
        this.employee_listeners.remove(FullDepartmentListener.class, l);
    }

    public void fireEmployeeAddededEvent(Employee e) {
        FullDepartmentListener[] listenersList =
                (FullDepartmentListener[]) 
                this.employee_listeners.getListeners(FullDepartmentListener.class);
        for (FullDepartmentListener l: listenersList) {
            l.employeeAdded(new FullDepartmentChangedEvent(this,e));
        }
    }
    
    public void fireEmployeeChangedEvent(Employee e) {
        FullDepartmentListener[] listenersList =
                (FullDepartmentListener[]) this.employee_listeners.getListeners(FullDepartmentListener.class);
        for (FullDepartmentListener l: listenersList) {
            l.employeeChanged(new FullDepartmentChangedEvent(this,e));
        }
    }
    
    public void fireEmployeeRemovedEvent(Employee e) {
        FullDepartmentListener[] listenersList =
                (FullDepartmentListener[]) this.employee_listeners.getListeners(FullDepartmentListener.class);
        for (FullDepartmentListener l: listenersList) {
            l.employeeRemoved(new FullDepartmentChangedEvent(this,e));
        }
    }

    @Override
    public void addEmployee(Employee e) {
        boolean exts = this.exists(e);
        super.addEmployee(e);
        if (exts) {
            System.out.println("change "+e.getName());
            this.fireEmployeeChangedEvent(e);
        }
        else {
            System.out.println("add "+e.getName());
            this.fireEmployeeAddededEvent(e);
        }
    }

    @Override
    public void removeEmployee(Employee e) {
        boolean suppressed = this.exists(e);
        System.out.println("remove "+e.getName());
        super.removeEmployee(e);
        this.fireEmployeeRemovedEvent(e);
    }
    
    
}
