package review.exercices.propertyeditor;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionListener;
import java.util.HashMap;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

public class PropertyEditorView extends JFrame implements ListSelectionListener {

    public static final String DEFAULT_KEY_LABEL = "Key";
    public static final String DEFAULT_VALUE_LABEL = "Value";
    public static final String DEFAULT_BTN_1_CMD = "Add";
    public static final String DEFAULT_BTN_2_CMD = "Change";
    public static final String DEFAULT_BTN_3_CMD = "Remove";
    private JLabel keyLabel;
    private JLabel valueLabel;
    private JTextField keyField;
    private JTextField valueField;
    private JList keyList;
    private JButton btn1;
    private JButton btn2;
    private JButton btn3;
    public DefaultListModel<KeyValuePair> data;

    public PropertyEditorView() {
        this.initComponent();
        this.placeComponent();
    }

    public void setButton1Action(String cmd) {
        this.btn1.setActionCommand(cmd);
        this.btn1.setText(cmd);
    }

    public String getButton1Action() {
        return this.btn1.getActionCommand();
    }

    public void setButton2Action(String cmd) {
        this.btn2.setActionCommand(cmd);
        this.btn2.setText(cmd);
    }

    public String getButton2Action() {
        return this.btn2.getActionCommand();
    }

    public void setButton3Action(String cmd) {
        this.btn2.setActionCommand(cmd);
        this.btn2.setText(cmd);
    }

    public String getButton3Action() {
        return this.btn2.getActionCommand();
    }

    public void addButtonListener(String cmd, ActionListener l) {
        if (cmd.equals(this.btn1.getActionCommand())) {
            this.btn1.addActionListener(l);
        }
        if (cmd.equals(this.btn2.getActionCommand())) {
            this.btn2.addActionListener(l);
        }
        if (cmd.equals(this.btn3.getActionCommand())) {
            this.btn3.addActionListener(l);
        }
    }

    public void removeButtonListener(String cmd, ActionListener l) {
        if (cmd.equals(this.btn1.getActionCommand())) {
            this.btn1.removeActionListener(l);
        }
        if (cmd.equals(this.btn2.getActionCommand())) {
            this.btn2.removeActionListener(l);
        }
        if (cmd.equals(this.btn3.getActionCommand())) {
            this.btn3.removeActionListener(l);
        }
    }

    public void addListSelectionListener(ListSelectionListener l) {
        this.keyList.addListSelectionListener(l);
    }

    public void removeListSelectionListener(ListSelectionListener l) {
        this.keyList.removeListSelectionListener(l);
    }

    public HashMap getSelectedElement() {
        HashMap m = new HashMap();
        int index = this.keyList.getSelectedIndex();
        if (index >= 0) {
            KeyValuePair p = this.data.get(index);
            m.put(p.getKey(), p.getValue());
        }
        return m;
    }

    public void addListElement(String k, String v) {
        KeyValuePair p = new KeyValuePair(k, v);
        this.data.addElement(p);
    }

    public void removeListElement(String k) {
        int index = -1;
        for (int i = 0; i < this.data.size(); i++) {
            KeyValuePair p = this.data.elementAt(i);
            if (k.equals(p.getKey())) {
                index = i;
            }
        }
        if (index > 0) {
            this.data.remove(index);
        }
    }

    public void setKeyLabel(String label) {
        this.keyLabel.setText(label);
    }

    public String getKeyLabel() {
        return this.keyLabel.getText();
    }

    public void setValueLabel(String label) {
        this.valueLabel.setText(label);
    }

    public String getValueLabel() {
        return this.valueLabel.getText();
    }

    public void setKeyFieldText(String text) {
        this.keyField.setText(text);
    }

    public String getKeyFieldText() {
        return this.keyField.getText();
    }

    public void setValueFieldText(String text) {
        this.valueField.setText(text);
    }

    public String getValueFieldText() {
        return this.valueField.getText();
    }

    @Override
    public void valueChanged(ListSelectionEvent e) {
        int index = this.keyList.getSelectedIndex();
        if (index > 0) {
            KeyValuePair p = this.data.get(index);
            if (p != null) {
                this.setKeyFieldText(p.getKey());
                this.setValueFieldText(p.getValue());
            }
        }
    }

        private void initComponent() {
        this.data = new DefaultListModel<KeyValuePair>();
        this.keyList = new JList(this.data);

        this.keyLabel = new JLabel(DEFAULT_KEY_LABEL);
        this.valueLabel = new JLabel(DEFAULT_VALUE_LABEL);

        this.keyField = new JTextField();
        this.keyField.setColumns(25);
        this.valueField = new JTextField();
        this.valueField.setColumns(25);

        this.btn1 = new JButton(DEFAULT_BTN_1_CMD);
        this.btn1.setActionCommand(DEFAULT_BTN_1_CMD);
        this.btn2 = new JButton(DEFAULT_BTN_2_CMD);
        this.btn2.setActionCommand(DEFAULT_BTN_2_CMD);
        this.btn3 = new JButton(DEFAULT_BTN_3_CMD);
        this.btn3.setActionCommand(DEFAULT_BTN_3_CMD);
    }
        
    private void placeComponent() {
        this.setLayout(new GridBagLayout());
        
        GridBagConstraints c = new GridBagConstraints();
        c.fill = GridBagConstraints.HORIZONTAL;
        c.gridx = 1;
        c.gridy = 0;
        this.add(this.keyLabel, c);
        
        c = new GridBagConstraints();
        c.fill = GridBagConstraints.HORIZONTAL;
        c.gridx = 1;
        c.gridy = 1;
        this.add(this.keyField, c);
        
        c = new GridBagConstraints();
        c.fill = GridBagConstraints.HORIZONTAL;
        c.gridx = 1;
        c.gridy = 2;
        this.add(this.valueLabel, c);
        
        c = new GridBagConstraints();
        c.fill = GridBagConstraints.HORIZONTAL;
        c.gridx = 1;
        c.gridy = 3;
        this.add(this.valueField, c);
        
        c = new GridBagConstraints();
        c.gridx = 1;
        c.gridy = 4;
        this.add(this.btn1, c);
        
        c = new GridBagConstraints();
        c.gridx = 2;
        c.gridy = 4;
        this.add(this.btn2, c);
        
        c = new GridBagConstraints();
        c.gridx = 3;
        c.gridy = 4;
        this.add(this.btn3, c);
        
        c = new GridBagConstraints();
        c.fill = GridBagConstraints.VERTICAL;
        c.anchor = GridBagConstraints.EAST;
        c.gridx = 0;
        c.gridy = 0;
        c.gridheight = 4;
        c.weighty = 0.5;
        JScrollPane scroll = new JScrollPane(this.keyList);
        this.add(scroll, c);        
    }
    

    private class KeyValuePair {

        String key;
        String value;

        public KeyValuePair(String k, String v) {
            key = k;
            value = v;
        }

        public String getKey() {
            return key;
        }

        public String getValue() {
            return value;
        }

        public HashMap toHash() {
            HashMap map = new HashMap();
            map.put(key, value);
            return map;
        }

        @Override
        public String toString() {
            return key;
        }
    }
}