package review.ihm.international;

import java.awt.event.ActionEvent;
import java.util.Locale;
import java.util.MissingResourceException;
import java.util.ResourceBundle;
import javax.swing.JOptionPane;

/**
 * exemple d'usage des ResourceBundle pour l'internationalisation d'une
 * applicaiton.
 *
 * @author kirsch
 */
public class ResourceBundleDemoFrame extends LocaleDemoFrame {
    public static final String DEFAULT_TITLE = "Resource Demo";
    public static final String LABEL_TITLE = "Title";
    public static final String LABEL_VALUES = "LabelValues";
    public static final String LABEL_LOCALES = "LabelLocales";
    public static final String ERROR_NO_RESOURCE = "No resource bundle available for this locale...";
    
    //le ResourceBundle est interprété comme une classe, donc on lui donne
    //le nom de la classe qui l'utilise, on lui met dans le classpath avec
    //cette classe et voilà, le tour est joué. 
    private String RESOURCE_BUNDLE = this.getClass().getName(); 
    
    public void loadResource(Locale locale) {
        ResourceBundle res = null;
                
        System.out.println("looking for bundle " + RESOURCE_BUNDLE + " locale " + locale);
            
        //on récupère d'abord le ResourceBundle
        try {
            if (locale != null) {
                res = ResourceBundle.getBundle(RESOURCE_BUNDLE, locale);
            } else {
                res = ResourceBundle.getBundle(RESOURCE_BUNDLE);
            }
        } catch (MissingResourceException mre) {
            JOptionPane.showMessageDialog(this, ERROR_NO_RESOURCE, "Alert", JOptionPane.ERROR_MESSAGE);
            mre.printStackTrace();
        }

         String title = "";
         String labelValues = ""; 
         String labelLocales = "";
        
         //puis on récupère les messages à afficher
         try {
             title = res.getString(LABEL_TITLE);
             labelValues = res.getString(LABEL_VALUES);
             labelLocales = res.getString(LABEL_LOCALES);
         } catch (MissingResourceException mre) {
            //si on n'a pas récupérer une ressource, on garde la valeur par défaut
         }
         
         if (!title.isEmpty())  this.setTitle(title); 
         if (!labelValues.isEmpty()) super.setValueLabel(labelValues); 
         if (!labelLocales.isEmpty())  super.setLocaleLabel(labelLocales); 
            
    }

    @Override
    public void actionPerformed(ActionEvent e) {
        this.loadResource(super.getSelectedLocale());
        super.actionPerformed(e);
    }
}