/*
 * UNIVERSITE PARIS 1 (PANTHEON SORBONNE)
 * MIAGE - UFR 27
 * Cours ISI5 / INF2 - Developpement d'Interface
 */
package review.ihm.volumeapplication;

import javax.swing.event.EventListenerList;

/**
 * Example d'usage du modele MVC
 * Cette classe definit le modele de donnees indicant un volume.
 *
 * Example base sur Baptiste Wicht, <b>"Implémentation du pattern MVC"</b>,
 * 24 Avril 2007 (derniere visite le 29 Janvier 2009).
 * @see http://baptiste-wicht.developpez.com/tutoriel/conception/mvc/
 * @author kirsch
 */
public class VolumeModel {

    public VolumeModel() {
        this(0);
    }

    public VolumeModel(int volume) {
        this.volume = volume;
        this.listeners = new EventListenerList();
    }

    /**
     * Cette methode permet de recuperer l'etat du modele, a savoir la valeur
     * du volume stocke.
     * @return int volume
     */
    public int getVolume() {
        return volume;
    }

    /**
     * Cette methode permet de modifier l'etat du modele, a savoir la valeur
     * du volume stocke.
     * @param volume nouvelle valeur de volume stocke
     */
    public void setVolume(int volume) {
        this.volume = volume;
        //on modifie l'etat du modele, on averti les listeners
        fireVolumeChanged();
    }

    /**
     * Methode utilisee pour abonner un nouveau listener a ce modele
     * @param listener nouveau listener
     */
    public void addVolumeListener(VolumeModelListener listener) {
        listeners.add(VolumeModelListener.class, listener);
    }

    /**
     * Methode utilisee pour desabonner un listener de ce modele
     * @param l listener qui veut se desabonner
     */
    public void removeVolumeListener(VolumeModelListener l) {
        listeners.remove(VolumeModelListener.class, l);
    }

    /**
     * methode declanchant la notification des tous les listeners suite a une
     * modification dans l'etat du modele.
     */
    protected void fireVolumeChanged() {
        VolumeModelListener[] listenerList =
                (VolumeModelListener[]) listeners.getListeners(VolumeModelListener.class);
        //foreach listener, on appelle la methode volumeChanged
        for (VolumeModelListener listener : listenerList) {
            listener.volumeChanged(new VolumeChangedEvent(this, this.getVolume()));
        }
    }

    //volume
    private int volume;
    //listeners
    private EventListenerList listeners;
}
