/*
 *  UNIVERSITE PARIS 1 (PANTHEON SORBONNE)
 *  MIAGE - UFR 27
 */
package review.serialization;

import review.serialization.PostIt;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * Exemple de sérialisation en Java. Puisque la classe PostIt mplémente 
 * serializable, on peut utiliser ses objets dans les flots de type 
 * ObjectOutputStream et ObjectInputStream afin de la sérialisé (ou pour 
 * transmettre sur le réseau, soit pour l'enregistrer dans un fichier, comme 
 * c'est le cas ici). 
 * 
 * @author kirsch
 */
public class PostItMain {
    private String filename = "postIt.dat";

    public String getFilename() {
        return filename;
    }

    public void setFilename(String filename) {
        this.filename = filename;
    }   
    
    public void doSerialize (PostIt msg) {
        try {
            ObjectOutputStream out = new ObjectOutputStream(new FileOutputStream(filename));
            out.writeObject(msg);
        } catch (IOException ex) {
            Logger.getLogger(PostIt.class.getName()).log(Level.SEVERE, "Impossible d'écrire sur le fichier", ex);
        }
    }

    public PostIt doDeserialize() {
        PostIt post = null;
        try {
            ObjectInputStream in = new ObjectInputStream(new FileInputStream(filename));
            post = (PostIt) in.readObject();
        } catch (IOException ex) {
            Logger.getLogger(PostIt.class.getName()).log(Level.SEVERE, "Impossible d'ouvrir le fichier", ex);
        } catch (ClassNotFoundException ex) {
            Logger.getLogger(PostIt.class.getName()).log(Level.SEVERE, "Classe PostIt inconnue", ex);
        }
        return post;
    }

    public static void main(String[] args) {
        PostItMain main = new PostItMain();
        main.setFilename("postIt.dat");
        
        PostIt post = new PostIt("This is a message in a bottle !!");
        main.doSerialize(post);
        
        PostIt post2 = main.doDeserialize();
        
        System.out.println(post2);
        System.out.println(post);
        System.out.println("Are they the same object ? " + (post.equals(post2)));
    }

    
}
