/*
 * UNIVERSITE PARIS 1 (PANTHEON SORBONNE)
 * MIAGE - UFR 27
 */
package review.structures;

/**
 * Classe représentant un tableau d'objets, dans lequel l'accès aux cases
 * est directe. 
 * Il s'agit d'un <b>mauvais exemple</b>, car il ne fait pas usage des 
 * génériques en Java. 
 * @author Manuele Kirsch Pinheiro <kirschpm@gmail.com>
 */
public class TableauObjetBugged {
    private static int DEFAULT_SIZE = 10; 
    private Object[] tableau;
    private int taille;
    private Class classe; //on va essayer de contrôler le type des objets

    /**
     * construction d'un objet TableauObjetBugged d'une taille donnée. Si celle-ci
     * ne correspond pas à une taille valable (size<=0 par exemple), le tableau
     * aura une taille par défaut. 
     * @param size taille du nouveau tableau d'objets
     */
    public TableauObjetBugged(int size) {
        if (size>0)
            this.taille = size;
        else
            this.taille = DEFAULT_SIZE;
        
        this.tableau = new Object[this.taille];
        this.classe = null; //en principe, on connaît pas le type des objets
    }

    /**
     * remplit la case dont l'indice est "pos" avec l'objet "value"
     * @param pos indice de la case à remplir
     * @param value valeur avec lequel on remplit la case
     */
    public void fillCase(int pos, Object value) {
        if (pos>=0 && pos<this.taille) {
            //code normal (sans s'occuper du type)
            this.tableau[pos] = value;
            
            /* a la place de la ligne ci-dessus,
             * on peut essayer de contrôler le type à utiliser, mais ça
             * devient vite très compliqué, comme on voit sur le code ci-dessous,
             * et ce n'est toujours pas garantit.
             */
            /*
             * //le premier objet inséré donne le type à utiliser
             * if (this.classe == null && value != null)
             *   this.classe = value.getClass();
             * //on n'insère l'objet que s'il est du bon type
             * if (value != null && value.getClass() == this.classe) {
             *    this.tableau[pos] = value;
             * 
             *  }
             */ 
        } 
    }

    /**
     * retourne la valeur d'une case occupée, ou null si la case est vide
     * @param pos indice de la case dont la valeur on recherche
     * @return Object stocké dans la case, null si la case est vide
     */
    public Object getCase(int pos) {
        if (pos>=0 && pos<this.taille)
            return this.tableau[pos];
        else 
            return null;
    }

    /**
     * nettoie la valeur d'une case
     * @param pos indice de la case à nettoyer
     */
    public void clearCase(int pos) {
        if (pos>=0 && pos<this.taille) {
            this.tableau[pos]=null;
        }
    }

    /**
     * taille du tableau
     * @return int taille déclarée du tableau
     */
    public int size() {
        return this.taille;
    }

    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder();
        sb.append("{ ");
        for (Object o : this.tableau) {
            sb.append(o);
            sb.append(' ');
        }
        sb.append('}');
        return sb.toString();
    }
    
    
}